==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/import.cc <==
#include <assert.h>
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

/*
	Copie les fichiers d'une projet et met  jour version.dat de ce projet
*/
static int near import_copy(
	const char *projet,
	const char *revstr,
	const REVISION *rev,
	USERINFO *user)
{
	int ret = 0;
	if (!file_exist("version.dat")){
		xconf_error (MSG_U(E_NOVERSIONDAT
			,"version.dat manquant pour importation\n"
			 "du sous-projet %s %s")
			,revstr,projet);
		ret = -1;
	}else{
		char *tb[400];
		int nb = dir_getlistp (WILD_ALLFILE,0,tb,400);
		for (int i=0; i<nb && ret != -1; i++){
			char *fname = tb[i];
			if (strcmp(fname,"version.dat")!=0){
				ret = foper_archive (tb[i],projet,rev,user);
			}
		}
		if (ret != -1){
			// Met  jour le version.dat
			VERSION_DAT version(user,projet);
			version.merge ("version.dat");
			version.save(user);
		}
		tbstr_free (tb,nb);
	}
	return ret;
}

/*
	Importe une livraison de /kit/livre dans /kit/ombre
*/
static int near import_one (
	const char *dirpath,	// Path du rpertoire livraison
							// /kit/livre ou /kit/sync/site
	const char *dirname,
	USERINFO *user)
{
	/* #Spcification: /kit/livre / importation / oprations
		Tous les fichiers d'une livraison sont simplement copi
		dans l'archive /kit/ombre  l'exception des fichiers
		version.dat. Ceux-ci sont fusionn avec les fichiers
		version.dat de /kit/ombre.

		Pour des raisons pratiques, l'organisation des rpertoires
		de /kit/livre et /kit/sync sont diffrents des rpertoires
		de /kit/ombre. Durant le copiage, il y aura la
		transformation de path suivante.

			/kit/livre/REV/path_projet/fichier
		ou	/kit/sync/site/REV/path_projet/fichier

		donnera

			/kit/ombre/path_projet/REV.s/fichier
	*/
	char livrepath[MAXSIZ_PATH];
	path_make (dirpath,dirname,livrepath);
	char *tbdir[400];
	int nb = dir_getlistd (livrepath,WILD_ALLFILE,100,tbdir,400);
	int len = strlen(livrepath) + 1;	// Saute le path et sparateur
	char revstr[MAXSIZ_REVISION];
	revision_path2rev (dirname,revstr);
	REVISION rev (revstr);
	int ret = 0;
	for (int i=0; i<nb && ret != -1; i++){
		char *prjpath = tbdir[i];
		assert ((int)strlen(prjpath)>len);
		char *projet = prjpath + len;
		SAVEPATH save;
		path_pushdir (user,prjpath,&save);
		ret = import_copy (projet,dirname,&rev,user);
		path_popdir (&save);
	}
	{
		SAVEPATH save;
		path_pushdir (user,livrepath,&save);
		ret = import_copy ("",dirname,&rev,user);
		path_popdir (&save);
	}
	tbstr_free (tbdir,nb);
	char vpath[MAXSIZ_PATH];
	path_make (dirpath,"version.dat",vpath);
	if (file_exist (vpath)){
		/* #Spcification: admin / import / site version.dat
			Si un rpertoire /kit/livre ou /kit/sync/site contient
			un version.dat, il sera mis a jour avec le version.dat principal
			de la rvision import. C'est comme ca qu'on sait qu'on doit
			pas rexport une rvision  un site qui nous l'a envoye.

			Le fichier version.dat est optionnel.
		*/
		VERSION_DAT ver(vpath);
		char rpath[MAXSIZ_PATH];
		path_make (livrepath,"version.dat",rpath);
		ver.merge (rpath);
		ver.save(user);
	}		
	return ret;
}

/*
	Localise toutes les rvisions dans /kit/livre et /kit/sync
	Retourne le nombre de rvisions plac dans tbdir (via strdup).
*/
static int near import_getrevdir (
	USERINFO *user,
	char *tbdir[],		// Contiendra les noms de sous-rpertoires
	char *tbpath[],		// Contiendra les paths (sans le nom) correspondant
	int maxdir)
{
	// On cherche d'abord dans /kit/livre
	const char *kitlivre = user->getlivre();
	int ret = projet_getrevdir(kitlivre,tbdir,maxdir);
	for (int i=0; i<ret; i++) tbpath[i] = strdup_err (kitlivre);
	char *tbs[100];
	int nbs = dir_getlistd (user->getsyncpath(),WILD_ALLFILE,0,tbs,100);
	for (int i=0; i<nbs; i++){
		int debut = ret;
		const char *sitedir = tbs[i];
		ret += projet_getrevdir (sitedir,tbdir+ret,maxdir-ret);
		for (int j=debut; j<ret; j++) tbpath[j] = strdup_err (sitedir);
	}
	return ret;
}

/* #Spcification: /kit/livre / importation
	Une copie de chaque livraison est plac dans le rpertoire
	/kit/livre. Cela permet de maintenir une copie de l'archive
	 diffrent endroit, sans avoir  tous copier.

	De plus, les rvisions que l'on recoit des sites satellites
	(qui ont aussi des developpeurs et un archive) sont plac
	dans les rpertoires correspondant /kit/sync/XXX ou XXX est
	le nom d'un site.

	A partir des sous-rpertoires de /kit/livre et de /kit/sync,
	on peut mettre  jour l'archive (rpertoire /kit/ombre).
	La sous-option "import" de l'option principale "admin" permet de
	slectivement mettre  jour l'archive.

	Les rpertoires /kit/livre et /kit/sync/XXX sont balays
	pour recueillir la liste des rvisions livrs qui ne sont pas
	dj dans l'archive. L'usager choisit les rvisions qu'il veut
	incorporer  l'archive et la fusion se ralise.
*/
/*
	Mise  jour de l'archive avec le contenu de /kit/livre et /kit/sync
*/
void import (
	USERINFO *user)
{
	/* #Spcification: /kit/livre / importation / partielle
		Par dfaut, toutes les rvisions sont imports. La possibilit
		de laisser de cot une rvision est ... une possibilit
		dangeureuse. Elle est disponible pour des raisons de flexibilit
		mais ne devrait tre utilis que par un utilisateurs
		avertis. Les raisons exactes motivant un tel comportement
		ne sont pas connues. :-)

		Les diffrentes rvisions sont souvent interdpendante.
		Si on importe par exemple, la rvision 0.9.4.2 et qu'on
		laisse de cot la rvision 0.9.4.1, on risque d'obtenir
		un archive incomplte. La version 0.9.4.2 fait probablement
		rfrence  des composantes de la rvision 0.9.4.1 qui
		seront absent de l'archive.

		Tous ceci n'est pas dangeureux (car les ordinateurs sont
		nos amis). L'ordre d'importation n'est pas importante.
	*/
	char *tbdir[400];
	char *tbpath[400];
	int nb = import_getrevdir(user,tbdir,tbpath,400);
	PAGE_EDIT page ("import");
	PAGE_EDIT spage ("s_import");
	page_setuptitre (&page,MSGTIT_IMPORT);
	char tb_is_sel[400];	// Est-ce que cette rvision est slectionne
	int tb_f2sel[400];		// Traduit un numero de champs -> tb_is_sel
	int au_moins_un = 0;
	{
		VERSION_DAT version (user);
		for (int i=0; i<nb; i++){
			char revision[MAXSIZ_REVISION];
			char *dir = tbdir[i];
			int ok = revision_path2rev(dir,revision);
			assert (ok!=-1);
			/* #Spcification: /kit/livre / entretien
				L'option importation n'efface pas le contenu
				de /kit/livre ni /kit/sync/XXX. Par contre, il n'est pas
				possible d'importer deux fois la mme
				rvision. Une rvision dj prsente
				est exclue de la slection offerte
				 l'usager.
			*/
			REVISION rev (revision);
			if (version.get(&rev)==NULL){
				// La rvision n'est pas dans l'archive.
				tb_is_sel[i] = 1;
				int nof = page_setupenumvc (&spage,-1,&tb_is_sel[i],1,"",1);
				tb_f2sel[nof] = i;
				page_setupfixestr0 (&spage,-1,revision,MAXSIZ_REVISION);
				au_moins_un = 1;
			}else{
				// Comme dj l (dans l'archive, ne sera pas
				// prsent et donc pas slectionnable et donc pas
				// slectionn.
				tb_is_sel[i] = 0;	
			}
		}
	}
	if (au_moins_un == 0){
		xconf_error (MSG_U(E_NOIMPORT,"Aucune rvision  importer"));
	}else{
		if (spage.nbfield() > 30) spage.setnbdisp(15);
		spage.linedesign0(2);
		page.setup (-1,&spage,0);
		bouton_setupstd (&page,"ok cancel");
		bouton_setupuser(&page,MSGBUT_ALL,K_NULL,PAGE_VAL_USER1);
		bouton_setupuser(&page,MSGBUT_NONE,K_NULL,PAGE_VAL_USER2);
		page.draw();
		while (1){
			PAGE_VAL val = page.edit(mskwin,cmd);
			if (cmd->is_load || val == PAGE_VAL_ABORT){
				break;
			}else if (val == PAGE_VAL_OK){
				page.save();
				WINDOW_LOG log("import.log",1);
				log.show();
				foper_setlog (&log);
				for (int i=0; i<nb; i++){
					if (tb_is_sel[i]){
						log.printf ("Importation rvision %s\n",tbdir[i]);
						if (import_one (tbpath[i],tbdir[i],user) == -1){
							break;
						}
					}
				}
				foper_setlog (NULL);
				break;
			}else{
				// Force toutes les slections  ON ou OFF
				char newval = val == PAGE_VAL_USER1;
				for (int i=0; i<page.nbfield(); i++){
					tb_is_sel[tb_f2sel[i]] = newval;
				}
				page.reload();
				for (i=0; i<page.nbfield(); i++){
					page.drawitem(i);
				}
			}	
		}
	}
	tbstr_free (tbdir,nb);
	tbstr_free (tbpath,nb);
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/import.cc <==
#include <assert.h>
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

/*
	Copie les fichiers d'une projet et met  jour version.dat de ce projet
*/
static int near import_copy(
	const char *projet,
	const char *revstr,
	const REVISION *rev,
	USERINFO *user)
{
	int ret = 0;
	if (!file_exist("version.dat")){
		xconf_error (MSG_U(E_NOVERSIONDAT
			,"version.dat manquant pour importation\n"
			 "du sous-projet %s %s")
			,revstr,projet);
		ret = -1;
	}else{
		char *tb[400];
		int nb = dir_getlistp (WILD_ALLFILE,0,tb,400);
		for (int i=0; i<nb && ret != -1; i++){
			char *fname = tb[i];
			if (strcmp(fname,"version.dat")!=0){
				ret = foper_archive (tb[i],projet,rev,user);
			}
		}
		if (ret != -1){
			// Met  jour le version.dat
			VERSION_DAT version(user,projet);
			version.merge ("version.dat");
			version.save(user);
		}
		tbstr_free (tb,nb);
	}
	return ret;
}

/*
	Importe une livraison de /kit/livre dans /kit/ombre
*/
static int near import_one (
	const char *dirpath,	// Path du rpertoire livraison
							// /kit/livre ou /kit/sync/site
	const char *dirname,
	USERINFO *user)
{
	/* #Spcification: /kit/livre / importation / oprations
		Tous les fichiers d'une livraison sont simplement copi
		dans l'archive /kit/ombre  l'exception des fichiers
		version.dat. Ceux-ci sont fusionn avec les fichiers
		version.dat de /kit/ombre.

		Pour des raisons pratiques, l'organisation des rpertoires
		de /kit/livre et /kit/sync sont diffrents des rpertoires
		de /kit/ombre. Durant le copiage, il y aura la
		transformation de path suivante.

			/kit/livre/REV/path_projet/fichier
		ou	/kit/sync/site/REV/path_projet/fichier

		donnera

			/kit/ombre/path_projet/REV.s/fichier
	*/
	char livrepath[MAXSIZ_PATH];
	path_make (dirpath,dirname,livrepath);
	char *tbdir[400];
	int nb = dir_getlistd (livrepath,WILD_ALLFILE,100,tbdir,400);
	int len = strlen(livrepath) + 1;	// Saute le path et sparateur
	char revstr[MAXSIZ_REVISION];
	revision_path2rev (dirname,revstr);
	REVISION rev (revstr);
	int ret = 0;
	for (int i=0; i<nb && ret != -1; i++){
		char *prjpath = tbdir[i];
		assert ((int)strlen(prjpath)>len);
		char *projet = prjpath + len;
		SAVEPATH save;
		path_pushdir (user,prjpath,&save);
		ret = import_copy (projet,dirname,&rev,user);
		path_popdir (&save);
	}
	{
		SAVEPATH save;
		path_pushdir (user,livrepath,&save);
		ret = import_copy ("",dirname,&rev,user);
		path_popdir (&save);
	}
	tbstr_free (tbdir,nb);
	char vpath[MAXSIZ_PATH];
	path_make (dirpath,"version.dat",vpath);
	if (file_exist (vpath)){
		/* #Spcification: admin / import / site version.dat
			Si un rpertoire /kit/livre ou /kit/sync/site contient
			un version.dat, il sera mis a jour avec le version.dat principal
			de la rvision import. C'est comme ca qu'on sait qu'on doit
			pas rexport une rvision  un site qui nous l'a envoye.

			Le fichier version.dat est optionnel.
		*/
		VERSION_DAT ver(vpath);
		char rpath[MAXSIZ_PATH];
		path_make (livrepath,"version.dat",rpath);
		ver.merge (rpath);
		ver.save(user);
	}		
	return ret;
}

/*
	Localise toutes les rvisions dans /kit/livre et /kit/sync
	Retourne le nombre de rvisions plac dans tbdir (via strdup).
*/
static int near import_getrevdir (
	USERINFO *user,
	char *tbdir[],		// Contiendra les noms de sous-rpertoires
	char *tbpath[],		// Contiendra les paths (sans le nom) correspondant
	int maxdir)
{
	// On cherche d'abord dans /kit/livre
	const char *kitlivre = user->getlivre();
	int ret = projet_getrevdir(kitlivre,tbdir,maxdir);
	for (int i=0; i<ret; i++) tbpath[i] = strdup_err (kitlivre);
	char *tbs[100];
	int nbs = dir_getlistd (user->getsyncpath(),WILD_ALLFILE,0,tbs,100);
	for (int i=0; i<nbs; i++){
		int debut = ret;
		const char *sitedir = tbs[i];
		ret += projet_getrevdir (sitedir,tbdir+ret,maxdir-ret);
		for (int j=debut; j<ret; j++) tbpath[j] = strdup_err (sitedir);
	}
	return ret;
}

/* #Spcification: /kit/livre / importation
	Une copie de chaque livraison est plac dans le rpertoire
	/kit/livre. Cela permet de maintenir une copie de l'archive
	 diffrent endroit, sans avoir  tous copier.

	De plus, les rvisions que l'on recoit des sites satellites
	(qui ont aussi des developpeurs et un archive) sont plac
	dans les rpertoires correspondant /kit/sync/XXX ou XXX est
	le nom d'un site.

	A partir des sous-rpertoires de /kit/livre et de /kit/sync,
	on peut mettre  jour l'archive (rpertoire /kit/ombre).
	La sous-option "import" de l'option principale "admin" permet de
	slectivement mettre  jour l'archive.

	Les rpertoires /kit/livre et /kit/sync/XXX sont balays
	pour recueillir la liste des rvisions livrs qui ne sont pas
	dj dans l'archive. L'usager choisit les rvisions qu'il veut
	incorporer  l'archive et la fusion se ralise.
*/
/*
	Mise  jour de l'archive avec le contenu de /kit/livre et /kit/sync
*/
void import (
	USERINFO *user)
{
	/* #Spcification: /kit/livre / importation / partielle
		Par dfaut, toutes les rvisions sont imports. La possibilit
		de laisser de cot une rvision est ... une possibilit
		dangeureuse. Elle est disponible pour des raisons de flexibilit
		mais ne devrait tre utilis que par un utilisateurs
		avertis. Les raisons exactes motivant un tel comportement
		ne sont pas connues. :-)

		Les diffrentes rvisions sont souvent interdpendante.
		Si on importe par exemple, la rvision 0.9.4.2 et qu'on
		laisse de cot la rvision 0.9.4.1, on risque d'obtenir
		un archive incomplte. La version 0.9.4.2 fait probablement
		rfrence  des composantes de la rvision 0.9.4.1 qui
		seront absent de l'archive.

		Tous ceci n'est pas dangeureux (car les ordinateurs sont
		nos amis). L'ordre d'importation n'est pas importante.
	*/
	char *tbdir[400];
	char *tbpath[400];
	int nb = import_getrevdir(user,tbdir,tbpath,400);
	DIALOG dia;
	char tb_is_sel[400];	// Est-ce que cette rvision est slectionne
	int tb_f2sel[400];		// Traduit un numero de champs -> tb_is_sel
	bool au_moins_un = false;
	{
		VERSION_DAT version (user);
		for (int i=0; i<nb; i++){
			char revision[MAXSIZ_REVISION];
			char *dir = tbdir[i];
			int ok = revision_path2rev(dir,revision);
			assert (ok!=-1);
			/* #Spcification: /kit/livre / entretien
				L'option importation n'efface pas le contenu
				de /kit/livre ni /kit/sync/XXX. Par contre, il n'est pas
				possible d'importer deux fois la mme
				rvision. Une rvision dj prsente
				est exclue de la slection offerte
				 l'usager.
			*/
			REVISION rev (revision);
			if (version.get(&rev)==NULL){
				// La rvision n'est pas dans l'archive.
				tb_is_sel[i] = 1;
				int nof = dia.getnb();
				dia.newf_chk ("",tb_is_sel[i],revision);
				tb_f2sel[nof] = i;
				au_moins_un = true;
			}else{
				// Comme dj l (dans l'archive, ne sera pas
				// prsent et donc pas slectionnable et donc pas
				// slectionn.
				tb_is_sel[i] = 0;	
			}
		}
	}
	if (!au_moins_un){
		xconf_error (MSG_U(E_NOIMPORT,"Aucune rvision  importer"));
	}else{
		dia.setbutinfo (MENU_USR1,MSG_R(B_ALL),MSG_R(B_ALL));
		dia.setbutinfo (MENU_USR2,MSG_R(B_NONE),MSG_R(B_NONE));
		int nof = 0;
		while (1){
			MENU_STATUS code = dia.edit(MSG_U(T_IMPORT,"Importation")
				,"",help_nil,nof);
			if (code == MENU_ESCAPE || code == MENU_CANCEL){
				break;
			}else if (code == MENU_ACCEPT){
				WINDOW_LOG log("import.log",1);
				log.show();
				foper_setlog (&log);
				for (int i=0; i<nb; i++){
					if (tb_is_sel[i]){
						log.printf ("Importation rvision %s\n",tbdir[i]);
						if (import_one (tbpath[i],tbdir[i],user) == -1){
							break;
						}
					}
				}
				foper_setlog (NULL);
				break;
			}else{
				// Force toutes les slections  ON ou OFF
				char newval = code == MENU_USR1;
				for (int i=0; i<dia.getnb(); i++){
					tb_is_sel[tb_f2sel[i]] = newval;
				}
				dia.reload();
			}	
		}
	}
	tbstr_free (tbdir,nb);
	tbstr_free (tbpath,nb);
}
